/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file component-tree.cpp
/// @brief Example program for QVision library. Gets component tree from images and prunes small sized low or high areas from images.
///
/// This program will receive a path or url to a video file, and then will prune low and high regions from the component tree of the image,
/// and show the resulting image.
/// To use the program, simply compile, and execute the binary:
///
///	./component-tree --Video=<path or url to video>
///
/// @author PARP Research Group. University of Murcia, Spain.
///

#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <QDebug>

#include <qvcore/qvapplication.h>
#include <qvmplayercamera/qvmplayercamera.h>
#include <qvgui/qvisioninterface.h>

#include <qvdta/qvdta.h>
#include <qvdta/qvcomponenttree.h>

class MyWorker: public QVWorker
	{
	public:
		MyWorker(QString name): QVWorker(name)
			{
			addProperty<int>("Maximal area to prune", inputFlag, 50,
				"Maximal size of the areas to be pruned in the image", 5, 320*240);
			addProperty< QVImage<uChar,1> >("Input image", inputFlag|outputFlag);
			addProperty< QVImage<uChar,1> >("Pruned component tree image", outputFlag);
			}

		void worker()
			{
			//////////////////
			// Read parameters
			uInt minAreaThreshold = getPropertyValue<int>("Maximal area to prune");
			QVImage<uChar> image = getPropertyValue< QVImage<uChar,1> >("Input image");

			uInt	rows = image.getRows(), cols = image.getCols();
			timeFlag("Read parameters");
		
			/////////////////
			// Prune low areas from image, with component tree
			QVImage<uChar> sinAreasMin = image;
			qvdta::QVComponentTree componentTreeLow(cols, rows, image);
			//qvdta::getComponentTree(image, componentTreeLow);
			timeFlag("Call to getComponentTree for low areas");

			componentTreeLow.pruneLowRegions(sinAreasMin/*, componentTreeLow*/, minAreaThreshold);
			timeFlag("Prune low areas from image");

			/////////////////
			// Inverse image
			QVImage<uChar> notImage = sinAreasMin;
			for (uInt col = 0; col < cols; col++)
				for (uInt row = 0; row < rows; row++)
					notImage(col, row) = 255 - notImage(col, row);
			timeFlag("Inverse image");
		
			/////////////////
			// Prune high areas from image, with component tree
			QVImage<uChar> sinAreasMax = sinAreasMin;
			qvdta::QVComponentTree componentTreeHigh(cols, rows, notImage);
			//qvdta::getComponentTree(notImage, componentTreeHigh);
			timeFlag("Call to getComponentTree for high areas");

			componentTreeHigh.pruneHighRegions(sinAreasMax/*, componentTreeHigh*/, minAreaThreshold);
			timeFlag("Prune high areas from image");
		
			/////////////////
			// Publish resulting images
			setPropertyValue< QVImage<uChar,1> >("Pruned component tree image",sinAreasMax);
			timeFlag("Publish resulting images");
			}
	};

int main(int argc, char *argv[])
	{
	QVApplication app(argc, argv,
		
		"Example program for QVision library. Gets component tree from images."

		);
	
	QVMPlayerCamera camera("Video");
	MyWorker worker("CT Worker");
	QVisionInterface interface;

	camera.link(&worker,"Input image");
	interface.setVisible(&worker,"Input image");
	interface.setVisible(&worker,"Pruned component tree image");

	return app.exec();
	}


	
