/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file corners.cpp
/// @brief Example program for QVision library. Applies corner detection over an input video.
///
/// This program will receive a path or url to a video file, and then will show hot points or corners of each frame in a
/// window. It detects corners where the minimal eigenvalue of the first derivatives matrix for a pixel is greater than a
/// Threshold value.
/// To use the program, simply compile, and execute the binary:
///
///	./corners --Video=<path or url to video>
///
/// @author PARP Research Group. University of Murcia, Spain.
///

#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <QDebug>

#include <qvcore/qvapplication.h>
#include <qvmplayercamera/qvmplayercamera.h>
#include <qvgui/qvisioninterface.h>

#include <qvdta/qvpolyline.h>
#include <qvdta/qvdta.h>

class MyWorker: public QVWorker
	{
	public:
		MyWorker(QString name): QVWorker(name)
			{
			addProperty("Window size", inputFlag, 10, "window size ", 1, 100);
			addProperty< QVImage<uChar,1> >("Input image", inputFlag|outputFlag);
			addProperty< QVImage<uChar,1> >("Corpusculos", outputFlag);
			addProperty< QVImage<uChar,3> >("Corners", outputFlag);
			}

		void worker()
			{
			int sizeMax = getPropertyValue<int>("Window size");
		
			QVImage<uChar> image = getPropertyValue< QVImage<uChar,1> >("Input image");
			uInt rows = image.getRows(), cols = image.getCols();
			timeFlag("grab Frame");
		
			////////////
			// MinEigenval
			QVImage<uChar> buffer;
			qvipp::MinEigenValGetBufferSize(image, buffer);
			QVImage<sFloat>	eigval(cols, rows);
		
			qvipp::MinEigenVal(image,eigval, buffer);
			QVImage<uChar,1> temp = eigval;
			qvipp::Mul(temp, temp, eigval);
			timeFlag("Corpusculos");
		
			qvipp::MulC(eigval, temp, 10);
			setPropertyValue< QVImage<uChar,1> >("Corpusculos", temp);
			timeFlag("Muestra corpusculos");
		
			///////////
			// LocalMax
			QVImage<uChar> hotpoints(cols, rows);
			qvdta::FilterLocalMax(eigval, hotpoints, sizeMax, sizeMax);
			timeFlag("FilterLocalMax");
		
			///////////
			// 1 to 3 chan
			QVImage<uChar,3> destino = image;
			timeFlag("Convert uChar to uChar,3");
		
			// Draw corners
			QList<QPoint> hotPoints;
		
			QVIMAGE_INIT_READ(uChar,hotpoints);
			for(uInt row = 0; row < hotpoints.getRows(); row++)
				for(uInt col = 0; col < hotpoints.getCols(); col++)
					if (QVIMAGE_PIXEL(hotpoints, col, row,0))
						hotPoints.append(QPoint(col, row));
			timeFlag("Find corners");
		
			drawPoints(hotPoints, destino);
			timeFlag("Draw corners");
		
			setPropertyValue< QVImage<uChar,3> >("Corners", destino);
			timeFlag("showing corners");
			}
	};

int main(int argc, char *argv[])
	{
	QVApplication app(argc, argv,
		
		"Example program for QVision library. Applies corner detection over an input video."

		);

	QVMPlayerCamera camera("Video");
	MyWorker worker("Corners Worker");
	QVisionInterface interface;

	camera.link(&worker,"Input image");
	interface.setVisible(&worker,"Input image");
	interface.setVisible(&worker,"Corpusculos");
	interface.setVisible(&worker,"Corners");

	return app.exec();
	}
