/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file parallel-threads.cpp
/// @brief Example program for QVision library. Shows how to construct a program using several processing threads.
///
/// This program will create four workers and launch them in separate threads, linking properties amonst them and 
/// setting data dependencies. The execution should show how properties sharing amonst workers is a thread-safe,
/// easy and clean way to develop multi-threaded programs with QVision.
///
/// To use the program, simply compile, and execute the binary:
///
///	./parallel-threads
///
/// @author PARP Research Group. University of Murcia, Spain.
///

#include <stdio.h>
#include <stdlib.h>
#include <iostream>

#include <qvcore/qvapplication.h>
#include <qvmplayercamera/qvmplayercamera.h>
#include <qvgui/qvisioninterface.h>

class MyWorker1: public QVWorker
	{
	public:
		MyWorker1(QString name):QVWorker(name)
			{
			addProperty<int>("int value", inputFlag|outputFlag, 10, "Integer test property", 10, 100);
			addProperty<double>("double value", inputFlag|outputFlag, 1.5, "Double test property", 0.0, 10.0);
			addProperty<bool>("boolean value", inputFlag|outputFlag, TRUE, "Boolean test property");
			}

		void worker()
			{
			int i,j=0;
			for(i=0;i<50000*2;i++) j=j+qrand();
			timeFlag("Tiempo 1");
			for(i=0;i<100000*2;i++) j=j+qrand();
			timeFlag("Tiempo 2");
			for(i=0;i<150000*2;i++) j=j+qrand();
			timeFlag("Tiempo 3");
			
			// Access to properties:
			int v1 = getPropertyValue<int>("int value");
			double v2 = getPropertyValue<double>("double value");
			bool v3 = getPropertyValue<bool>("boolean value");
			std::cout << "\tint value= " << v1 << "\tdouble value = " << v2 << "\tboolean value = " << v3 << "\n";
			}
	};

class MyWorker2: public QVWorker
	{
	private:
		int id;

	public:
		MyWorker2(QString name,int _id):QVWorker(name), id(_id)
			{
			addProperty<int>("int value", inputFlag|outputFlag, 60, "Integer test property", 50, 70);
			addProperty<int>(QString("i%1").arg(id), outputFlag|inputFlag,15, "Integer test property", 10, 100);
			addProperty<double>(QString("d%1").arg(id), outputFlag|inputFlag, 1.5, "Double test property", 0.0, 10.0);
			addProperty<bool>(QString("b%1").arg(id), outputFlag|inputFlag, TRUE, "Boolean test property");
			}

		void worker()
			{
			int i,j=0;
			for(i=0;i<50000*2*id;i++) j=j+qrand();
			timeFlag("Tiempo 1");

			// Access to properties:
			int vi = getPropertyValue<int>(QString("i%1").arg(id));
			double vd = getPropertyValue<double>(QString("d%1").arg(id));
			bool vb = getPropertyValue<bool>(QString("b%1").arg(id));

			QString msg;
			for(int i=0;i<id;i++) 
				msg += "\t\t";

			msg +=  QString("i%1=%2").arg(id).arg(vi) + QString("\td%1=%2").arg(id).arg(vd) + QString("\tb%1=%2 ").arg(id).arg(vb) + "\n";
			std::cout << qPrintable(msg);
			}
	};

int main(int argc, char *argv[])
	{
	QVApplication app(argc, argv,

		"Example program for QVision library. Shows the use of QVApllication and QVWorker classes."

		);

	MyWorker1 worker1("W1");
	MyWorker2 worker2("W2",2), worker3("W3",3), worker4("W4",4);

	worker1.linkProperty("double value", &worker2, "d2", QVWorker::SynchronousLink);
	worker1.linkProperty("int value", &worker3, "i3", QVWorker::SynchronousLink);
	worker1.linkProperty("double value", &worker3, "d3", QVWorker::SynchronousLink);
	worker1.linkProperty("boolean value", &worker3, "b3", QVWorker::SynchronousLink);
	worker1.linkProperty("boolean value", &worker4, "b4", QVWorker::AsynchronousLink);

	QVisionInterface interface;

	return app.exec();
	}
