/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvparamwidget.cpp
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#include <qvgui/qvparamwidget.h>

QVIntParamWidget::QVIntParamWidget(QVPropertyHolder * propertyHolder, const QString property, QWidget *parent): QWidget(parent)
	{
	Q_ASSERT_X(propertyHolder->containsProperty(property),
		"QVDoubleParamWidget::QVDoubleParamWidget()", "Property doesn't exists in the given property holder");
	Q_ASSERT_X(propertyHolder->getPropertyType(property) == QVariant::Int,
		"QVDoubleParamWidget::QVDoubleParamWidget()", "Property not int typed");

	this->propertyHolder = propertyHolder;
	this->paramName = property;
	this->max = propertyHolder->getPropertyMaximum<int>(property);
	this->min = propertyHolder->getPropertyMinimum<int>(property);
	int value = propertyHolder->getPropertyValue<int>(property);

	form.setupUi(this);
	form.label_3->setText(paramName);
	form.spinBox->setRange(min, max);
	form.spinBox->setValue(value);
	setMinimumHeight(50);
	connect(form.spinBox,SIGNAL(valueChanged(int)), this, SLOT(setValue(int)));
	}

void QVIntParamWidget::setValue(int value)
	{
	Q_ASSERT_X(value <= this->max, "QVIntParamWidget::setValue", "value > maximum");
	Q_ASSERT_X(value >= this->min, "QVIntParamWidget::setValue", "value > minimum");
	propertyHolder->setPropertyValue<int>(paramName,value);
	}

QVDoubleParamWidget::QVDoubleParamWidget(QVPropertyHolder * propertyHolder, const QString property, QWidget *parent): QWidget(parent)
	{
	Q_ASSERT_X(propertyHolder->containsProperty(property),
		"QVDoubleParamWidget::QVDoubleParamWidget()", "Property doesn't exists in the given property holder");
	Q_ASSERT_X(propertyHolder->getPropertyType(property) == QVariant::Double,
		"QVDoubleParamWidget::QVDoubleParamWidget()", "Property not double typed");

	this->propertyHolder = propertyHolder;
	this->paramName = property;
	this->max = propertyHolder->getPropertyMaximum<double>(property);
	this->min = propertyHolder->getPropertyMinimum<double>(property);
	this->fraction = 100/(max-min);
	double value = propertyHolder->getPropertyValue<double>(property);

	form.setupUi(this);
	form.label_2->setText(paramName);
	form.label->setText(QString().setNum(form.horizontalSlider->value()));
	form.horizontalSlider->setRange((int)(min*fraction), (int)(max*fraction));
	form.horizontalSlider->setValue((int)(value*fraction));
	form.label->setText(QString().setNum(value));

	setMinimumHeight(50);
	QObject::connect(form.horizontalSlider, SIGNAL(valueChanged(int)), this, SLOT(updateValue(int)));
	}

void QVDoubleParamWidget::updateValue(int value)
	{
	double doubleValue = ((double)value)/fraction;
	form.label->setText(QString().setNum(doubleValue));
	setValue(doubleValue);
	}

void QVDoubleParamWidget::setValue(double value)
	{
	Q_ASSERT_X(value <= this->max, "QVDoubleParamWidget::setValue", "value > maximum");
	Q_ASSERT_X(value >= this->min, "QVDoubleParamWidget::setValue", "value < minimum");
	propertyHolder->setPropertyValue<double>(paramName,value);
	}

QVBoolParamWidget::QVBoolParamWidget(QVPropertyHolder * propertyHolder, const QString property, QWidget *parent): QCheckBox(parent)
	{
	Q_ASSERT_X(propertyHolder->containsProperty(property),
		"QVDoubleParamWidget::QVDoubleParamWidget()", "Property doesn't exists in the given property holder");
	Q_ASSERT_X(propertyHolder->getPropertyType(property) == QVariant::Bool,
		"QVDoubleParamWidget::QVDoubleParamWidget()", "Property not bool typed");

	this->propertyHolder = propertyHolder;
	this->paramName = property;
	setText(paramName);

	bool value = propertyHolder->getPropertyValue<bool>(property);

	connect(this,SIGNAL(stateChanged(int)), this,SLOT(stateChangedSlot(int)));
	setCheckState(value?Qt::Checked:Qt::Unchecked);
	}

void QVBoolParamWidget::stateChangedSlot(int state)
	{
	if (state == Qt::Unchecked)
		setValue(false);
	else if (state == Qt::Checked)
		setValue(true);
	}

void QVBoolParamWidget::setValue(bool value)
	{
	propertyHolder->setPropertyValue<bool>(paramName,value);
	}




