/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvipp.h
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#ifndef QVIPP_H
#define QVIPP_H

#include <qvcore/qvimage.h>
#include <QMap>

namespace qvipp
{
/// @brief Copies pixel values between two images.
///
/// This function copies data from a source image ROI src to the destination image ROI dest.
///
/// @param src source image.
/// @param dest destination image.
void Copy(const QVImage<uChar, 1> &src, QVImage<uChar, 1> &dest);

/// @brief Copies pixel values between two images.
/// @see Copy(const QVImage<uChar, 1> &, QVImage<uChar, 1> &)
void Copy(const QVImage<sShort, 1> &src, QVImage<sShort, 1> &dest);

/// @brief Copies pixel values between two images.
/// @see Copy(const QVImage<uChar, 1> &, QVImage<uChar, 1> &)
void Copy(const QVImage<sFloat, 1> &src, QVImage<sFloat, 1> &dest);

/// @brief Copies pixel values between two images.
/// @see Copy(const QVImage<uChar, 1> &, QVImage<uChar, 1> &)
void Copy(const QVImage<uChar, 3> &src, QVImage<uChar, 3> &dest);

/// @brief Copies pixel values between two images.
/// @see Copy(const QVImage<uChar, 1> &, QVImage<uChar, 1> &)
void Copy(const QVImage<sShort, 3> &src, QVImage<sShort, 3> &dest);

/// @brief Copies pixel values between two images.
/// @see Copy(const QVImage<uChar, 1> &, QVImage<uChar, 1> &)
void Copy(const QVImage<sFloat, 3> &src, QVImage<sFloat, 3> &dest);

/// @brief Copies pixel values from a 1-channel image to all of the channels for a 3-channel image.
///
/// This function copies data from a source 1-channel image ROI src, to each of the channels of the destination 3-channel image ROI 
/// dest.
///
/// @param src source image.
/// @param dest destination image.
void Copy(const QVImage<uChar, 1> &src, QVImage<uChar, 3> &dest);

/// @brief Copies pixel values from a 1-channel image to all of the channels for a 3-channel image.
/// @see Copy(const QVImage<uChar, 1> &, QVImage<uChar, 3> &)
void Copy(const QVImage<sShort, 1> &src, QVImage<sShort, 3> &dest);

/// @brief Copies pixel values from a 1-channel image to all of the channels for a 3-channel image.
/// @see Copy(const QVImage<uChar, 1> &, QVImage<uChar, 3> &)
void Copy(const QVImage<sFloat, 1> &src, QVImage<sFloat, 3> &dest);

/// @brief Compares pixel values of two images using a specified compare operation.
///
/// This function compares the corresponding pixels of ROI in two source images src1, src2 using the ippCmpOp compare operation, 
/// and writes the results to a 1-channel uChar image dest.
///
/// If the result of the compare is true, the corresponding output pixel is set to an IPP_MAX_8U value; otherwise, it is set to 0.
///
/// @param src1 first operand in comparison image.
/// @param src2 second operand in comparison image.
/// @param dest image to store comparison value pixels.
/// @param ippCmpOp Compare operation to be used for comparing the pixel values. Must be one of these enumerated values:
///	- ippCmpLess,		for less comparison
///	- ippCmpLessEq,		for less or equal comparison
///	- ippCmpEq,		for equal comparison
///	- ippCmpGreaterEq,	for greater or equal comparison
///	- ippCmpGreater,	for greater comparison
void Compare(const QVImage<uChar> &src1, const QVImage<uChar> &src2, QVImage<uChar> &dest, IppCmpOp ippCmpOp = ippCmpEq);

/// @brief Compares pixel values of two images using a specified compare operation.
/// @see void Compare(const QVImage<uChar> &, const QVImage<uChar> &src2, QVImage<uChar> &, IppCmpOp)
void Compare(const QVImage<sShort> &src1, const QVImage<sShort> &src2, QVImage<uChar> &dest, IppCmpOp ippCmpOp = ippCmpEq);

/// @brief Compares pixel values of two images using a specified compare operation.
/// @see void Compare(const QVImage<uChar> &, const QVImage<uChar> &src2, QVImage<uChar> &, IppCmpOp)
void Compare(const QVImage<sFloat> &src1, const QVImage<sFloat> &src2, QVImage<uChar> &dest, IppCmpOp ippCmpOp = ippCmpEq);

/// @brief Compares pixel values of two images using a specified compare operation.
/// @see void Compare(const QVImage<uChar> &, const QVImage<uChar> &src2, QVImage<uChar> &, IppCmpOp)
void Compare(const QVImage<uChar,3> &src1, const QVImage<uChar,3> &src2, QVImage<uChar> &dest, IppCmpOp ippCmpOp = ippCmpEq);

/// @brief Compares pixel values of two images using a specified compare operation.
/// @see void Compare(const QVImage<uChar> &, const QVImage<uChar> &src2, QVImage<uChar> &, IppCmpOp)
void Compare(const QVImage<sShort,3> &src1, const QVImage<sShort,3> &src2, QVImage<uChar> &dest, IppCmpOp ippCmpOp = ippCmpEq);

/// @brief Compares pixel values of two images using a specified compare operation.
/// @see void Compare(const QVImage<uChar> &, const QVImage<uChar> &src2, QVImage<uChar> &, IppCmpOp)
void Compare(const QVImage<sFloat,3> &src1, const QVImage<sFloat,3> &src2, QVImage<uChar> &dest, IppCmpOp ippCmpOp = ippCmpEq);

/// @brief Compares pixel values of a source image to a given value using a specified compare operation.
///
/// This function compares pixels of the each channel of the source image ROI src to a given
/// value specified for each channel using the ippCmpOp compare operation, and writes the
/// results to a 1-channel Ipp8u image dest.
///
/// If the result of the compare is true, that is all pixels of all channels satisfy to the specified condition,
/// then the corresponding output pixel is set to an IPP_MAX_8U value; otherwise, it is set to 0.
///
/// @param src image to compare pixel values.
/// @param dest image to store pixel comparison values.
/// @param constant value to compare each pixel value.
/// @param ippCmpOp Compare operation to be used for comparing the pixel values. Must be one of these enumerated values:
///	- ippCmpLess,		for less comparison
///	- ippCmpLessEq,		for less or equal comparison
///	- ippCmpEq,		for equal comparison
///	- ippCmpGreaterEq,	for greater or equal comparison
///	- ippCmpGreater,	for greater comparison
void CompareC(const QVImage<uChar> &src, QVImage<uChar> &dest, uChar constant, IppCmpOp ippCmpOp = ippCmpEq);

/// @brief Compares pixel values of a source image to a given value using a specified compare operation.
/// @see CompareC(const QVImage<uChar> &, QVImage<uChar> &, uChar, IppCmpOp ippCmpOp)
void CompareC(const QVImage<sFloat> &src, QVImage<uChar> &dest, sFloat constant, IppCmpOp ippCmpOp = ippCmpEq);

/// @brief Converts a image from one bit depth to another.
///
/// This function converts pixel values in the source image ROI pSrc to a different bit depth
/// and writes them to the destination image ROI pDst. No scaling is done.
///
/// @param src image to convert bit depth.
/// @param dest image to store resulting bit depth image conversion.
void Convert(const QVImage<uChar, 1> &src, QVImage<sShort, 1> &dest);

/// @brief Converts a image from one bit depth to another.
/// @see Convert(const QVImage<uChar, 1> &, QVImage<sShort, 1> &)
void Convert(const QVImage<uChar, 1> &src, QVImage<sFloat, 1> &dest);

/// @brief Converts a image from one bit depth to another.
/// @see Convert(const QVImage<uChar, 1> &, QVImage<sShort, 1> &)
void Convert(const QVImage<sShort, 1> &src, QVImage<uChar, 1> &dest);

/// @brief Converts a image from one bit depth to another.
/// @see Convert(const QVImage<uChar, 1> &, QVImage<sShort, 1> &)
void Convert(const QVImage<sShort, 1> &src, QVImage<sFloat, 1> &dest);

/// @brief Converts a image from one bit depth to another.
/// @see Convert(const QVImage<uChar, 1> &, QVImage<sShort, 1> &)
void Convert(const QVImage<uChar, 3> &src, QVImage<sShort, 3> &dest);

/// @brief Converts a image from one bit depth to another.
/// @see Convert(const QVImage<uChar, 1> &, QVImage<sShort, 1> &)
void Convert(const QVImage<uChar, 3> &src, QVImage<sFloat, 3> &dest);

/// @brief Converts a image from one bit depth to another.
/// @see Convert(const QVImage<uChar, 1> &, QVImage<sShort, 1> &)
void Convert(const QVImage<sShort, 3> &src, QVImage<uChar, 3> &dest);

/// @brief Converts a image from one bit depth to another.
/// @see Convert(const QVImage<uChar, 1> &, QVImage<sShort, 1> &)
void Convert(const QVImage<sShort, 3> &src, QVImage<sFloat, 3> &dest);

/// @brief Converts a image from a floating point bit depth to a integer bit depth, using a given round mode.
///
/// This function converts pixel values in the source image ROI pSrc to a different bit depth
/// and writes them to the destination image ROI pDst. No scaling is done.
///
/// Rounding defined by roundMode is performed, and the result is saturated to the destination data type range.
///
/// @param src image to convert bit depth.
/// @param dest image to store resulting bit depth image conversion.
/// @param roundMode specified round mode. Possible values for the enumeration are:
///	- ippRndZero. Specifies that floating-point values must be truncated toward zero.
///	- ippRndNear. Specifies that floating-point values must be rounded to the nearest integer.
void Convert(const QVImage<sFloat, 1> &src, QVImage<uChar, 1> &dest, IppRoundMode roundMode = ippRndNear);

/// @brief Converts a image from a floating point bit depth to a integer bit depth, using a given round mode.
/// @see Convert(const QVImage<sFloat, 1> &, QVImage<uChar, 1> &, IppRoundMode)
void Convert(const QVImage<sFloat, 1> &src, QVImage<sShort, 1> &dest, IppRoundMode roundMode = ippRndNear);

/// @brief Converts a image from a floating point bit depth to a integer bit depth, using a given round mode.
/// @see Convert(const QVImage<sFloat, 1> &, QVImage<uChar, 1> &, IppRoundMode)
void Convert(const QVImage<sFloat, 3> &src, QVImage<uChar, 3> &dest, IppRoundMode roundMode = ippRndNear);

/// @brief Converts a image from a floating point bit depth to a integer bit depth, using a given round mode.
/// @see Convert(const QVImage<sFloat, 1> &, QVImage<uChar, 1> &, IppRoundMode)
void Convert(const QVImage<sFloat, 3> &src, QVImage<sShort, 3> &dest, IppRoundMode roundMode = ippRndNear);

/// @brief Converts an RGB image to gray-scale using fixed transform coefficients.
///
/// This function converts an RGB image to gray scale (see Figure 6-18) using the following basic equation to compute luma
/// from nonlinear gamma-corrected red, green, and blue values:
/// \verbatim Y = 0.299 * R + 0.587 * G + 0.114 * B \endverbatim
///
/// Note that the transform coefficients conform to the standard for the NTSC red, green, and blue CRT phosphors.
///
/// @param src RGB image to convert to gray scale.
/// @param dest resulting gray-scale image.
void RGBToGray(const QVImage<uChar, 3> &src, QVImage<uChar, 1> &dest);

/// @brief Converts an RGB image to gray-scale using fixed transform coefficients.
/// @see RGBToGray(const QVImage<uChar, 3> &, QVImage<uChar, 1> &)
void RGBToGray(const QVImage<sShort, 3> &src, QVImage<sShort, 1> &dest);

/// @brief Converts an RGB image to gray-scale using fixed transform coefficients.
/// @see RGBToGray(const QVImage<uChar, 3> &, QVImage<uChar, 1> &)
void RGBToGray(const QVImage<sFloat, 3> &src, QVImage<sFloat, 1> &dest);

/// @brief Performs thresholding of pixel values in an image buffer.
///
/// This function thresholds pixels in the source image src using the specified level threshold.
/// Pixel values in the source image are compared to the threshold value using the ippCmpOp comparison operation.
///
/// If the result of the compare is true, the corresponding output pixel is set to the threshold value.
/// Otherwise, it is set to the source pixel value.
///
/// @param src image to threshold.
/// @param dest image to store thresholded image.
/// @param constant value to compare each pixel value.
/// @param ippCmpOp Compare operation to be used for thresholding the pixel values. Must be one of these enumerated values:
///	- ippCmpLess,		for less comparison
///	- ippCmpLessEq,		for less or equal comparison
///	- ippCmpEq,		for equal comparison
///	- ippCmpGreaterEq,	for greater or equal comparison
///	- ippCmpGreater,	for greater comparison
void Threshold(const QVImage<uChar> &src, QVImage<uChar> &dest, uChar constant, IppCmpOp ippCmpOp = ippCmpEq);

/// @brief Performs thresholding of pixel values in an image buffer.
/// @see Threshold(const QVImage<uChar> &, QVImage<uChar> &, uChar, IppCmpOp ippCmpOp)
void Threshold(const QVImage<sShort> &src, QVImage<sShort> &dest, sShort constant, IppCmpOp ippCmpOp = ippCmpEq);

/// @brief Performs thresholding of pixel values in an image buffer.
/// @see Threshold(const QVImage<uChar> &, QVImage<uChar> &, uChar, IppCmpOp ippCmpOp)
void Threshold(const QVImage<sFloat> &src, QVImage<sFloat> &dest, sFloat constant, IppCmpOp ippCmpOp = ippCmpEq);

/// @brief Sets pixels of an image ROI to a value.
///
/// This function sets pixels in the destination image ROI img to a constant value.
///
/// @param img destination image to set.
/// @param constant value to set pixels in image ROI.
void Set(QVImage<uChar> &img, uChar constant);

/// @brief Sets pixels of an image ROI to a value.
/// @see Set(QVImage<uChar> &, uChar)
void Set(QVImage<sShort> &img, sShort constant);

/// @brief Sets pixels of an image ROI to a value.
/// @see Set(QVImage<uChar> &, uChar)
void Set(QVImage<sFloat> &img, sFloat constant);

/// @brief Sets pixels of a 3-channel image ROI to a value respective constant value, one for each channel.
///
/// This function sets pixels in the destination 3-channel image ROI img to a constant value. Each channel can be set to
/// a different constant value, with a different parameter value.
///
/// @param img destination image to set.
/// @param red value to set pixels in first channel of image ROI.
/// @param green value to set pixels in second channel of image ROI.
/// @param blue value to set pixels in third channel of image ROI.
void Set(QVImage<uChar, 3> &img, uChar red, uChar green, uChar blue);

/// @brief Sets pixels of a 3-channel image ROI to a value respective constant value, one for each channel.
/// @see Set(QVImage<uChar, 3> &, uChar, uChar, uChar)
void Set(QVImage<sShort, 3> &img, sShort red, sShort green, sShort blue);

/// @brief Sets pixels of a 3-channel image ROI to a value respective constant value, one for each channel.
/// @see Set(QVImage<uChar, 3> &, uChar, uChar, uChar)
void Set(QVImage<sFloat, 3> &img, sFloat red, sFloat green, sFloat blue);

/// @brief Computes absolute pixel values of a source image and places them into the destination image.
///
/// @param src source image.
/// @param dest destination image.
void Abs(const QVImage<sShort> &src, QVImage<sShort> &dest);

/// @brief Computes absolute pixel values of a source image and places them into the destination image.
/// @see Not(const QVImage<sShort> &, QVImage<sShort> &)
void Abs(const QVImage<sFloat> &src, QVImage<sFloat> &dest);

// boolean
/// @brief Performs a bit-wise NOT operation on each pixel of an image.
///
/// @param src source image to perform bit-wise NOT operation on.
/// @param dest destination image to hold result of bit-wise operation.
void Not(const QVImage<uChar> &src, QVImage<uChar> &dest);

/// @brief Performs a bit-wise NOT operation on each pixel of an image.
/// @see Not(const QVImage<uChar> &, QVImage<uChar> &)
void Not(const QVImage<uChar,3> &src, QVImage<uChar,3> &dest);

/// @brief Performs a bit-wise inclusive OR operation between each pixel of an image and a constant.
///
/// @param src source image to perform bit-wise inclusive OR operation on.
/// @param dest destination image to hold result of bit-wise operation.
/// @param constant value to perform bit-wise operation with.
void OrC(const QVImage<uChar> &src, QVImage<uChar> &dest, uChar constant);

/// @brief Performs a bit-wise AND operation between each pixel of an image and a constant.
///
/// @param src source image to perform bit-wise AND operation on.
/// @param dest destination image to hold result of bit-wise operation.
/// @param constant value to perform bit-wise operation with.
void AndC(const QVImage<uChar> &src, QVImage<uChar> &dest, uChar constant);

/// @brief Performs a bit-wise exclusive OR operation between each pixel of an image and a constant.
///
/// @param src source image to perform bit-wise exclusive OR operation on.
/// @param dest destination image to hold result of bit-wise operation.
/// @param constant value to perform bit-wise operation with.
void XorC(const QVImage<uChar> &src, QVImage<uChar> &dest, uChar constant);

/// @brief Shifts bits in pixel values to the left.
///
/// @param src source image to perform left shift on.
/// @param dest destination image to hold result of left shift.
/// @param constant value to perform left shift with.
void LShiftC(const QVImage<uChar> &src, QVImage<uChar> &dest, uChar constant);

/*/// @brief Shifts bits in pixel values to the left.
/// @see Shifty(const QVImage<uChar> &, QVImage<uChar> &, uChar)
void LShiftC(const QVImage<uShort> &src, QVImage<uShort> &dest, uChar constant);*/

/// @brief Shifts bits in pixel values to the right.
///
/// @param src source image to perform right shift on.
/// @param dest destination image to hold result of right shift.
/// @param constant value to perform right shift with.
void RShiftC(const QVImage<uChar> &src, QVImage<uChar> &dest, uChar constant);

/*/// @brief Computes the exponential of pixel values in a image.
///
/// @param src source image to perform operation.
/// @param dest destination image to write result.
/// @param constant value to perform exponential.
void ExpC(const QVImage<sShort> &src, QVImage<sShort> &dest, uChar constant);*/


/*/// @brief Shifts bits in pixel values to the left.
/// @see RShiftC(const QVImage<uChar> &, QVImage<uChar> &, uChar)
void RShiftC(const QVImage<uShort> &src, QVImage<uShort> &dest, uChar constant);*/

// aritmetic
/// @brief Adds pixel values of two images.
///
/// This function adds corresponding pixel values of two source image buffers and places the
/// results in a destination buffer. Saturation and scaling is used for integer data (uChar, sShort, etc...).
/// 
/// Scaling is done by multiplying the output pixel values by 2<sup>-scale</sup> before the function returns.
/// This helps retain either the output data range or its precision.
/// Usually the scaling with a positive factor is performed by the shift operation.
/// Then, the result is rounded off to the nearest integer number.
///
/// @param src1 source image to use pixels as first operands in add.
/// @param src2 source image to use pixels as second operands in add.
/// @param dest destination image to hold result pixels of add.
/// @param scale value to perform scaling of resulting pixel values, for integer bit depth.
void Add(const QVImage<uChar> &src1, const QVImage<uChar> &src2, QVImage<uChar> &dest, int scale = 1);

/// @brief Adds pixel values of two images.
/// @see Add(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Add(const QVImage<uChar,3> &src1, const QVImage<uChar,3> &src2, QVImage<uChar,3> &dest, int scale = 1);

/// @brief Adds pixel values of two images.
/// @see Add(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Add(const QVImage<sShort> &src1, const QVImage<sShort> &src2, QVImage<sShort> &dest, int scale = 1);

/// @brief Adds pixel values of two images.
/// @see Add(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Add(const QVImage<sShort,3> &src1, const QVImage<sShort,3> &src2, QVImage<sShort,3> &dest, int scale = 1);

/// @brief Adds pixel values of two images.
/// @see Add(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Add(const QVImage<sFloat> &src1, const QVImage<sFloat> &src2, QVImage<sFloat> &dest);

/// @brief Adds pixel values of two images.
/// @see Add(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Add(const QVImage<sFloat,3> &src1, const QVImage<sFloat,3> &src2, QVImage<sFloat,3> &dest);

/// @brief Subtracts pixel values of two images.
///
/// This function substracts corresponding pixel values of two source image buffers and places the
/// results in a destination buffer. Saturation and scaling is used for integer data (uChar, sShort, etc...).
/// 
/// Scaling is done by multiplying the output pixel values by 2<sup>-scale</sup> before the function returns.
/// This helps retain either the output data range or its precision.
/// Usually the scaling with a positive factor is performed by the shift operation.
/// Then, the result is rounded off to the nearest integer number.
///
/// @param src1 source image to use pixels as first operands in subtraction.
/// @param src2 source image to use pixels as second operands in subtraction.
/// @param dest destination image to hold result pixels of subtraction.
/// @param scale value to perform scaling of resulting pixel values, for integer bit depth.
void Sub(const QVImage<uChar> &src1, const QVImage<uChar> &src2,QVImage<uChar> &dest, int scale = 1);

/// @brief Subtracts pixel values of two images.
/// @see Sub(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Sub(const QVImage<uChar,3> &src1, const QVImage<uChar,3> &src2,QVImage<uChar,3> &dest, int scale = 1);

/// @brief Subtracts pixel values of two images.
/// @see Sub(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Sub(const QVImage<sShort> &src1, const QVImage<sShort> &src2,QVImage<sShort> &dest, int scale = 1);

/// @brief Subtracts pixel values of two images.
/// @see Sub(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Sub(const QVImage<sShort,3> &src1, const QVImage<sShort,3> &src2,QVImage<sShort,3> &dest, int scale = 1);

/// @brief Subtracts pixel values of two images.
/// @see Sub(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Sub(const QVImage<sFloat> &src1, const QVImage<sFloat> &src2,QVImage<sFloat> &dest);

/// @brief Subtracts pixel values of two images.
/// @see Sub(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Sub(const QVImage<sFloat,3> &src1, const QVImage<sFloat,3> &src2,QVImage<sFloat,3> &dest);

/// @brief Multiplies pixel values of two images.
///
/// This function multiplies corresponding pixel values of two source image buffers and places the
/// results in a destination buffer. Saturation and scaling is used for integer data (uChar, sShort, etc...).
/// 
/// Scaling is done by multiplying the output pixel values by 2<sup>-scale</sup> before the function returns.
/// This helps retain either the output data range or its precision.
/// Usually the scaling with a positive factor is performed by the shift operation.
/// Then, the result is rounded off to the nearest integer number.
///
/// @param src1 source image to use pixels as first operands in multiplication.
/// @param src2 source image to use pixels as second operands in multiplication.
/// @param dest destination image to hold result pixels of multiplication.
/// @param scale value to perform scaling of resulting pixel values, for integer bit depth.
void Mul(const QVImage<uChar> &src1, const QVImage<uChar> &src2,QVImage<uChar> &dest, int scale = 1);

/// @brief Multiplies pixel values of two images.
/// @see Mul(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Mul(const QVImage<uChar,3> &src1, const QVImage<uChar,3> &src2,QVImage<uChar,3> &dest, int scale = 1);

/// @brief Multiplies pixel values of two images.
/// @see Mul(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Mul(const QVImage<sShort> &src1, const QVImage<sShort> &src2,QVImage<sShort> &dest, int scale = 1);

/// @brief Multiplies pixel values of two images.
/// @see Mul(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Mul(const QVImage<sShort,3> &src1, const QVImage<sShort,3> &src2,QVImage<sShort,3> &dest, int scale = 1);

/// @brief Multiplies pixel values of two images.
/// @see Mul(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Mul(const QVImage<sFloat> &src1, const QVImage<sFloat> &src2,QVImage<sFloat> &dest);

/// @brief Multiplies pixel values of two images.
/// @see Mul(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Mul(const QVImage<sFloat,3> &src1, const QVImage<sFloat,3> &src2,QVImage<sFloat,3> &dest);

/// @brief Divides pixel values of an image by pixel values of another image.
///
/// This function divides corresponding pixel values of two source image buffers and places the
/// results in a destination buffer. Saturation and scaling is used for integer data (uChar, sShort, etc...).
/// 
/// Scaling is done by multiplying the output pixel values by 2<sup>-scale</sup> before the function returns.
/// This helps retain either the output data range or its precision.
/// Usually the scaling with a positive factor is performed by the shift operation.
/// Then, the result is rounded off to the nearest integer number.
///
/// @param src1 source image to use pixels as dividends in division.
/// @param src2 source image to use pixels as divisors in division.
/// @param dest destination image to hold result pixels of division.
/// @param scale value to perform scaling of resulting pixel values, for integer bit depth.
void Div(const QVImage<uChar> &src1, const QVImage<uChar> &src2,QVImage<uChar> &dest, int scale = 1);

/// @brief Divides pixel values of an image by pixel values of another image.
/// @see Div(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Div(const QVImage<uChar,3> &src1, const QVImage<uChar,3> &src2,QVImage<uChar,3> &dest, int scale = 1);

/// @brief Divides pixel values of an image by pixel values of another image.
/// @see Div(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Div(const QVImage<sShort> &src1, const QVImage<sShort> &src2,QVImage<sShort> &dest, int scale = 1);

/// @brief Divides pixel values of an image by pixel values of another image.
/// @see Div(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Div(const QVImage<sShort,3> &src1, const QVImage<sShort,3> &src2,QVImage<sShort,3> &dest, int scale = 1);

/// @brief Divides pixel values of an image by pixel values of another image.
/// @see Div(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Div(const QVImage<sFloat> &src1, const QVImage<sFloat> &src2,QVImage<sFloat> &dest);

/// @brief Divides pixel values of an image by pixel values of another image.
/// @see Div(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &, int)
void Div(const QVImage<sFloat,3> &src1, const QVImage<sFloat,3> &src2,QVImage<sFloat,3> &dest);

/// @brief Multiplies pixel values of two images and scales the products.
///
/// This function multiplies corresponding pixel values of two input buffers and scales the
/// products using the following formula:
/// \verbatim dest(i,j) = src1(i,j) * src2(i,j) / max_val \endverbatim
///
/// where max_val is the maximum value of the pixel data range.
///
/// @param src1 source image to use pixels as first operands in multiplication.
/// @param src2 source image to use pixels as second operands in multiplication.
/// @param dest destination image to hold result pixels of multiplication.
void MulScale(const QVImage<uChar> &src1, const QVImage<uChar> &src2, QVImage<uChar> &dest);

/// @brief Multiplies pixel values of two images and scales the products.
/// @see MulScale(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &)
void MulScale(const QVImage<uChar,3> &src1, const QVImage<uChar,3> &src2, QVImage<uChar,3> &dest);

/// @brief Calculates absolute difference between two images.
///
/// This function calculates the absolute pixel-wise difference between two images by the formula:
/// \verbatim dest(x,y) = abs(src1(x,y) - src2(x,y)) \endverbatim
///
/// @param src1 source image to use pixels as first operands in subtraction.
/// @param src2 source image to use pixels as second operands in subtraction.
/// @param dest destination image to hold result pixels of operation.
void AbsDiff(const QVImage<uChar> &src1, const QVImage<uChar> &src2,QVImage<uChar> &dest);

/// @brief Calculates absolute difference between two images.
/// @see AbsDiff(const QVImage<uChar> &, const QVImage<uChar> &, QVImage<uChar> &)
void AbsDiff(const QVImage<sFloat> &src1, const QVImage<sFloat> &src2,QVImage<sFloat> &dest);

// constant aritmetic
/// @brief Multiplies pixel values of an image by a constant.
///
/// This function multiplies pixel values of an image by a constant value.
/// Saturation and scaling is used for integer data (uChar, sShort, etc...).
/// 
/// Scaling is done by multiplying the output pixel values by 2<sup>-scale</sup> before the function returns.
/// This helps retain either the output data range or its precision.
/// Usually the scaling with a positive factor is performed by the shift operation.
/// Then, the result is rounded off to the nearest integer number.
///
/// @param src source image to use pixels as first operands in multiplication.
/// @param dest destination image to hold result pixels of multiplication.
/// @param constant value to use as second operand in multiplication.
/// @param scale value to perform scaling of resulting pixel values, for integer bit depth.
void MulC(const QVImage<uChar> &src, QVImage<uChar> &dest, uChar constant, int scale = 1);

/// @brief Multiplies pixel values of an image by a constant.
/// @see MulC(const QVImage<uChar> &, QVImage<uChar> &, uChar constant, int)
void MulC(const QVImage<sShort> &src, QVImage<sShort> &dest, sShort constant, int scale = 1);

/// @brief Multiplies pixel values of an image by a constant.
/// @see MulC(const QVImage<uChar> &, QVImage<uChar> &, uChar constant, int)
void MulC(const QVImage<sFloat> &src, QVImage<sFloat> &dest, sFloat constant);

/// @brief Adds a constant to pixel values of an image.
///
/// This function changes the image intensity by adding the value to image pixel values.
/// Saturation and scaling is used for integer data (uChar, sShort, etc...).
/// 
/// Scaling is done by multiplying the output pixel values by 2<sup>-scale</sup> before the function returns.
/// This helps retain either the output data range or its precision.
/// Usually the scaling with a positive factor is performed by the shift operation.
/// Then, the result is rounded off to the nearest integer number.
///
/// @param src source image to use pixels as first operands in add.
/// @param dest destination image to hold result pixels of add.
/// @param constant value to use as second operand in add.
/// @param scale value to perform scaling of resulting pixel values, for integer bit depth.
void AddC(const QVImage<uChar> &src, QVImage<uChar> &dest, uChar constant, int scale = 1);

/// @brief Adds a constant to pixel values of an image.
/// @see AddC(const QVImage<uChar> &, QVImage<uChar> &, uChar, int scale)
void AddC(const QVImage<sShort> &src, QVImage<sShort> &dest, sShort constant, int scale = 1);

/// @brief Adds a constant to pixel values of an image.
/// @see AddC(const QVImage<uChar> &, QVImage<uChar> &, uChar, int scale)
void AddC(const QVImage<sFloat> &src, QVImage<sFloat> &dest, sFloat constant);

/// @brief Subtracts a constant from pixel values of an image.
///
/// This function changes image intensity by subtracting the constant value from pixel values
/// of an image buffer.
/// Saturation and scaling is used for integer data (uChar, sShort, etc...).
/// 
/// Scaling is done by multiplying the output pixel values by 2<sup>-scale</sup> before the function returns.
/// This helps retain either the output data range or its precision.
/// Usually the scaling with a positive factor is performed by the shift operation.
/// Then, the result is rounded off to the nearest integer number.
///
/// @param src source image to use pixels as first operands in subtraction.
/// @param dest destination image to hold result pixels of subtraction.
/// @param constant value to use as second operand in subtraction.
/// @param scale value to perform scaling of resulting pixel values, for integer bit depth.
void SubC(const QVImage<uChar> &src, QVImage<uChar> &dest, uChar constant, int scale = 1);

/// @brief Subtracts a constant from pixel values of an image.
/// @see void SubC(const QVImage<uChar> &, QVImage<uChar> &, uChar, int)
void SubC(const QVImage<sShort> &src, QVImage<sShort> &dest, sShort constant, int scale = 1);

/// @brief Subtracts a constant from pixel values of an image.
/// @see void SubC(const QVImage<uChar> &, QVImage<uChar> &, uChar, int)
void SubC(const QVImage<sFloat> &src, QVImage<sFloat> &dest, sFloat constant);

/// @brief Divides pixel values of an image by a constant.
///
/// This function changes image intensity by dividing pixel values of an image buffer by the
/// constant value.
/// Saturation and scaling is used for integer data (uChar, sShort, etc...).
/// 
/// Scaling is done by multiplying the output pixel values by 2<sup>-scale</sup> before the function returns.
/// This helps retain either the output data range or its precision.
/// Usually the scaling with a positive factor is performed by the shift operation.
/// Then, the result is rounded off to the nearest integer number.
///
/// @param src source image to use pixels as first operands in multiplication.
/// @param dest destination image to hold result pixels of multiplication.
/// @param constant value to use as second operand in multiplication.
/// @param scale value to perform scaling of resulting pixel values, for integer bit depth.
void DivC(const QVImage<uChar> &src, QVImage<uChar> &dest, uChar constant, int scale = 1);

/// @brief Divides pixel values of an image by a constant.
/// @see DivC(const QVImage<uChar> &, QVImage<uChar> &, uChar, int)
void DivC(const QVImage<sShort> &src, QVImage<sShort> &dest, sShort constant, int scale = 1);

/// @brief Divides pixel values of an image by a constant.
/// @see DivC(const QVImage<uChar> &, QVImage<uChar> &, uChar, int)
void DivC(const QVImage<sFloat> &src, QVImage<sFloat> &dest, sFloat constant);

// statistics
/// @brief Computes the sum of image pixel values.
///
/// @param src source image to compute pixel sum.
/// @param value return value containing the sum of the values of the pixels of the image.
void Sum(const QVImage<uChar> &src, sDouble & value);

/// @brief Computes the sum of image pixel values.
/// @see Sum(const QVImage<uChar> &, sDouble &)
void Sum(const QVImage<sShort> &src, sDouble & value);

/// @brief Computes the sum of image pixel values, by separate planes.
///
/// @param src source image to compute pixel sum.
/// @param red return value containing the sum of the values of the pixels of the image for the first channel.
/// @param green return value containing the sum of the values of the pixels of the image for the second channel.
/// @param blue return value containing the sum of the values of the pixels of the image for the third channel.
void Sum(const QVImage<uChar,3> &src, sDouble & r, sDouble & g, sDouble & b);

/// @brief Computes the sum of image pixel values, by separate planes.
/// @see Sum(const QVImage<uChar,3> &, sDouble &, sDouble &, sDouble &)
void Sum(const QVImage<sShort,3> &src, sDouble & r, sDouble & g, sDouble & b);

/// @brief Computes the mean of image pixel values.
///
/// @param src source image to compute pixel mean.
/// @param value return value containing the mean of the values of the pixels of the image.
void Mean(const QVImage<uChar> &src, sDouble & value);

/// @brief Computes the mean of image pixel values.
/// @see Mean(const QVImage<uChar> &, sDouble &)
void Mean(const QVImage<sShort> &src, sDouble & value);

/// @brief Computes the mean of image pixel values, by separate planes.
///
/// @param src source image to compute pixel mean.
/// @param red return value containing the mean of the values of the pixels of the image for the first channel.
/// @param green return value containing the mean of the values of the pixels of the image for the second channel.
/// @param blue return value containing the mean of the values of the pixels of the image for the third channel.
void Mean(const QVImage<uChar,3> &src, sDouble & r, sDouble & g, sDouble & b);

/// @brief Computes the mean of image pixel values, by separate planes.
/// @see Mean(const QVImage<uChar,3> &, sDouble &, sDouble &, sDouble &)
void Mean(const QVImage<sShort,3> &src, sDouble & r, sDouble & g, sDouble & b);

/// @brief Computes the maximum of image pixel values.
///
/// @param src source image to compute pixel maximum.
/// @param value return value containing the maximum of the values of the pixels of the image.
void Max(const QVImage<uChar> &src, uChar & value);

/// @brief Computes the maximum of image pixel values.
/// @see Max(const QVImage<uChar> &, uChar &)
void Max(const QVImage<sShort> &src, sShort & value);

/// @brief Computes the maximum of image pixel values.
/// @see Max(const QVImage<uChar> &, uChar &)
void Max(const QVImage<sFloat> &src, sFloat & value);

/// @brief Computes the maximum of image pixel values, by separate planes.
///
/// @param src source image to compute pixel maximum.
/// @param red return value containing the maximum of the values of the pixels of the image for the first channel.
/// @param green return value containing the maximum of the values of the pixels of the image for the second channel.
/// @param blue return value containing the maximum of the values of the pixels of the image for the third channel.
void Max(const QVImage<uChar,3> &src, uChar & r, uChar & g, uChar & b);

/// @brief Computes the maximum of image pixel values, by separate planes.
/// @see Max(const QVImage<uChar,3> &, uChar &, uChar &, uChar &)
void Max(const QVImage<sShort,3> &src, sShort &r, sShort &g, sShort &b);

/// @brief Computes the minimum of image pixel values.
///
/// @param src source image to compute pixel minimum.
/// @param value return value containing the minimum of the values of the pixels of the image.
void Min(const QVImage<uChar> &src, uChar & value);

/// @brief Computes the minimum of image pixel values.
/// @see Min(const QVImage<uChar> &, uChar &)
void Min(const QVImage<sShort> &src, sShort & value);

/// @brief Computes the minimum of image pixel values.
/// @see Min(const QVImage<uChar> &, uChar &)
void Min(const QVImage<sFloat> &src, sFloat & value);

/// @brief Computes the minimum of image pixel values, by separate planes.
///
/// @param src source image to compute pixel minimum.
/// @param red return value containing the minimum of the values of the pixels of the image for the first channel.
/// @param green return value containing the minimum of the values of the pixels of the image for the second channel.
/// @param blue return value containing the minimum of the values of the pixels of the image for the third channel.
void Min(const QVImage<uChar,3> &src, uChar & r, uChar & g, uChar & b);

/// @brief Computes the minimum of image pixel values, by separate planes.
/// @see Min(const QVImage<uChar,3> &, uChar &, uChar &, uChar &)
void Min(const QVImage<sShort,3> &src, sShort &r, sShort &g, sShort &b);

// Filters
/// @brief Applies the <i>max</i> filter to an image.
///
/// This function sets each pixel in the destination image to the maximum value of all the
/// source image pixel values in the neighborhood of mask size given.
///
/// This has the effect of increasing the contrast in the image.
///
/// @param src source image to filter.
/// @param dest destination image to hold the resulting image after filtering.
/// @param colMaskSize mask width.
/// @param rowMaskSize mask height.
void FilterMax(const QVImage<uChar> &src, QVImage<uChar> &dest, uInt colMaskSize, uInt rowMaskSize);

/// @brief Applies the <i>max</i> filter to an image.
/// @see FilterMax(const QVImage<uChar> &, QVImage<uChar> &, uInt, uInt)
void FilterMax(const QVImage<sFloat> &src, QVImage<sFloat> &dest, uInt colMaskSize, uInt rowMaskSize);

/// @brief Applies the <i>min</i> filter to an image.
///
/// This function sets each pixel in the destination image to the minimum value of all the
/// source image pixel values in the neighborhood of mask size given.
///
/// This has the effect of decreasing the contrast in the image.
///
/// @param src source image to filter.
/// @param dest destination image to hold the resulting image after filtering.
/// @param colMaskSize mask width.
/// @param rowMaskSize mask height.
void FilterMin(const QVImage<uChar> &src, QVImage<uChar> &dest, uInt colMaskSize, uInt rowMaskSize);

/// @brief Applies the <i>min</i> filter to an image.
/// @see FilterMin(const QVImage<uChar> &, QVImage<uChar> &, uInt, uInt)
void FilterMin(const QVImage<sFloat> &src, QVImage<sFloat> &dest, uInt colMaskSize, uInt rowMaskSize);

/// @brief Blurs an image using a simple box filter.
///
/// This function sets each pixel in the output image as the average of all the input image
/// pixels in the rectangular neighborhood of size mask size given.
///
/// This has the effect of smoothing or blurring the input image.
///
/// @param src source image to filter.
/// @param dest destination image to hold the resulting image after filtering.
/// @param colMaskSize mask width.
/// @param rowMaskSize mask height.
void FilterBox(const QVImage<uChar> &src, QVImage<uChar> &dest, uInt colMaskRadius, uInt rowMaskRadius);

/// @brief Blurs an image using a simple box filter.
/// @see FilterBox(const QVImage<uChar> &, QVImage<uChar> &, uInt, uInt)
void FilterBox(const QVImage<sFloat> &src, QVImage<sFloat> &dest, uInt colMaskRadius, uInt rowMaskRadius);

/// @brief Filters an image using a Gaussian kernel.
///
/// This function applies a lowpass Gaussian filter to an image ROI.
/// The corresponding kernel is the matrix of either 3x3 or 5x5 size.
/// The 3x3 filter uses the kernel:
///
/// \f$ \left| \begin{array}{ccc} 1/16 & 2/16 & 1/16 \\ 2/16 & 4/16 & 2/16 \\ 1/16 & 2/16 & 1/16 \\ \end{array} \right| \f$
///
/// These filter coefficients correspond to a 2-dimensional Gaussian distribution with standard
/// deviation 0.85.
///
/// The 5x5 filter uses the kernel:
///
/// \f$ \left| \begin{array}{ccccc} 2/571 & 7/571 & 12/571 & 7/571 & 2/571 \\ 7/571 & 31/571 & 52/571 & 31/571 & 7/571 \\ 12/571 & 52/571 & 127/571 & 52/571 & 12/571 \\ 7/571 & 31/571 & 52/571 & 31/571 & 7/571 \\ 2/571 & 7/571 & 12/571 & 7/571 & 2/571 \\ \end{array} \right| \f$
///
/// These filter coefficients correspond to a 2-dimensional Gaussian distribution with standard
/// deviation 1.0.
///
/// @param src source image to filter.
/// @param dest destination image to hold the resulting image after filtering.
/// @param maskSize size of mask. Should be 3 (for mask of size 3x3) or 5 (for mask of size 5x5).
void FilterGauss(const QVImage<uChar> &src, QVImage<uChar> &dest, uInt maskSize);

/// @brief Filters an image using a Gaussian kernel.
/// @see FilterGauss(const QVImage<uChar> &, QVImage<uChar> &, uInt)
void FilterGauss(const QVImage<sFloat> &src, QVImage<sFloat> &dest, uInt maskSize);

/// @brief Filters an image using a high-pass filter.
///
/// This function applies a high-pass filter to an image ROI.
/// The corresponding kernel is the matrix of either 3x3 or 5x5 size with the following values:
///
/// \f$ \left| \begin{array}{ccc} -1 & -1 & -1 \\ -1 & +8 & -1 \\ -1 & -1 & -1 \\ \end{array} \right| \f$
///
/// for 3x3 size, or
///
/// \f$ \left| \begin{array}{ccccc} -1 & -1 & -1 & -1 & -1 \\ -1 & -1 & -1 & -1 & -1 \\ -1 & -1 & +24 & -1 & -1 \\ -1 & -1 & -1 & -1 & -1 \\ -1 & -1 & -1 & -1 & -1 \\ \end{array} \right| \f$
///
/// For 5x5 size. This filter attenuates low-frequency components and thus sharpens an image.
///
/// @param src source image to filter.
/// @param dest destination image to hold the resulting image after filtering.
/// @param maskSize size of mask. Should be 3 (for mask of size 3x3) or 5 (for mask of size 5x5).
void FilterHipass(const QVImage<uChar> &src, QVImage<uChar> &dest, uInt maskSize = 3);

/// @brief Filters an image using a high-pass filter.
/// @see FilterHipass(const QVImage<uChar> &, QVImage<uChar> &, uInt)
void FilterHipass(const QVImage<sFloat> &src, QVImage<sFloat> &dest, uInt maskSize = 3);

/// @brief Filters an image using a lowpass filter.
///
/// This filter blurs an image by averaging the pixel values over some neighborhood.
/// It works similar to FilterBox function, but uses two fixed mask sizes: 3x3, or 5x5.
///
/// @param src source image to filter.
/// @param dest destination image to hold the resulting image after filtering.
/// @param maskSize size of mask. Should be 3 (for mask of size 3x3) or 5 (for mask of size 5x5).
void FilterLowpass(const QVImage<uChar> &src, QVImage<uChar> &dest, uInt maskSize = 3);

/// @brief Filters an image using a lowpass filter.
/// @see FilterLowpass(const QVImage<uChar> &, QVImage<uChar> &, uInt)
void FilterLowpass(const QVImage<sFloat> &src, QVImage<sFloat> &dest, uInt maskSize = 3);

/// @brief Filters an image using a sharpening filter.
///
/// This function applies a sharpening filter to an image ROI.
/// The corresponding kernel is the matrix of 3x3 size with the following values:
///
/// \f$ \left| \begin{array}{ccc} -1/8 & -1/8 & -1/8 \\ -1/8 & 16/8 & -1/8 \\ -1/8 & -1/8 & -1/8 \\ \end{array} \right| \f$
///
/// This filter enhances high-frequency components and thus sharpens an image.
///
/// @param src source image to filter.
/// @param dest destination image to hold the resulting image after filtering.
void FilterSharpen(const QVImage<uChar> &src, QVImage<uChar> &dest);

/// @brief Filters an image using a sharpening filter.
/// @see FilterSharpen(const QVImage<uChar> &, QVImage<uChar> &)
void FilterSharpen(const QVImage<sFloat> &src, QVImage<sFloat> &dest);

/// @brief Filters an image using a horizontal Sobel kernel.
///
/// This function apply a horizontal Sobel operator to an image ROI.
///
/// This filter has the effect of both enhancing and smoothing horizontal edges of an image.
///
/// Warning: this function is not working properly. It will be repaired, for now better use function @ref FilterSobelHorizMask
///
/// @param src source image to filter.
/// @param dest destination image to hold the resulting image after filtering.
void FilterSobelHoriz(const QVImage<uChar> &src, QVImage<uChar> &dest);

/// @brief Filters an image using a horizontal Sobel kernel.
/// @see FilterSobelHoriz(const QVImage<uChar> &, QVImage<uChar> &)
void FilterSobelHoriz(const QVImage<sShort> &src, QVImage<sShort> &dest);

/// @brief Filters an image using a horizontal Sobel kernel.
/// @see FilterSobelHoriz(const QVImage<uChar> &, QVImage<uChar> &)
void FilterSobelHoriz(const QVImage<sFloat> &src, QVImage<sFloat> &dest);

/// @brief Filters an image using a vertical Sobel kernel.
///
/// This function apply a vertical Sobel operator to an image ROI.
///
/// This filter has the effect of both enhancing and smoothing vertical edges of an image.
///
/// Warning: this function is not working properly. It will be repaired, for now better use function @ref FilterSobelVertMask
///
/// @param src source image to filter.
/// @param dest destination image to hold the resulting image after filtering.
void FilterSobelVert(const QVImage<uChar> &src, QVImage<uChar> &dest);

/// @brief Filters an image using a vertical Sobel kernel.
/// @see FilterSobelVert(const QVImage<uChar> &, QVImage<uChar> &)
void FilterSobelVert(const QVImage<sShort> &src, QVImage<sShort> &dest);

/// @brief Filters an image using a vertical Sobel kernel.
/// @see FilterSobelVert(const QVImage<uChar> &, QVImage<uChar> &)
void FilterSobelVert(const QVImage<sFloat> &src, QVImage<sFloat> &dest);

/// @brief Filters an image using a horizontal Sobel kernel.
///
/// This function apply a horizontal Sobel operator to an image ROI.
///
/// The appropriate kernel is the matrix of 3x3 size, or either 3x3 or 5x5 size
/// in accordance with mask parameter of the corresponding function flavors.
/// The kernels have the following values:
///
/// \f$ \left| \begin{array}{ccc} 1 & 2 & 1 \\ 0 & 0 & 0 \\ -1 & -2 & -1 \\ \end{array} \right| \f$
///
/// for 3x3 size, or
///
/// \f$ \left| \begin{array}{ccccc} 1 & 4 & 6 & 4 & 1 \\ 2 & 8 & 12 & 8 & 2 \\ 0 & 0 & 0 & 0 & 0 \\ -2 & -8 & -12 & -8 & -2 \\ -1 & -4 & -6 & -4 & -1 \\ \end{array} \right| \f$
///
/// For 5x5 size. This filter has the effect of both enhancing and smoothing horizontal edges of an image.
///
/// @param src source image to filter.
/// @param dest destination image to hold the resulting image after filtering.
/// @param maskSize size of mask. Should be 3 (for mask of size 3x3) or 5 (for mask of size 5x5).
void FilterSobelHorizMask(const QVImage<sFloat> &src, QVImage<sFloat> &dest, uInt maskSize = 3);

/// @brief Filters an image using a vertical Sobel kernel.
///
/// This function apply a vertical Sobel operator to an image ROI. The appropriate kernel is the
/// matrix of 3x3 size, or either 3x3 or 5x5 size in accordance with mask parameter of the
/// corresponding function flavors. The kernels have the following values:
///
/// \f$ \left| \begin{array}{ccc} -1 & 0 & 1 \\ -2 & 0 & 2 \\ -1 & 0 & 1 \\ \end{array} \right| \f$
///
/// for 3x3 size, or
///
/// \f$ \left| \begin{array}{ccccc} -1 & -2 & 0 & 2 & 1 \\ -4 & -8 & 0 & 8 & 4 \\ -6 & -12 & 0 & 12 & 6 \\ -4 & -8 & 0 & 8 & 4 \\ -1 & -2 & 0 & 2 & 1 \\ \end{array} \right| \f$
///
/// For 5x5 size. This filter has the effect of both enhancing and smoothing vertical edges of an image.
///
/// @param src source image to filter.
/// @param dest destination image to hold the resulting image after filtering.
/// @param maskSize size of mask. Should be 3 (for mask of size 3x3) or 5 (for mask of size 5x5).
void FilterSobelVertMask(const QVImage<sFloat> &src, QVImage<sFloat> &dest, uInt maskSize = 3);

/// @brief Applies horizontal Sobel filter with border.
///
/// This function is similar to FilterSobelHorizMask, but automatically creates a required border and
/// define appropriate pixel values.
///
/// Function FilterSobelHorizGetBufferSize should be used to initialize sizes for a buffer QVImage<uChar>
/// image, which will hold temporary data for the function operations.
///
/// @param src source image to filter.
/// @param dest destination image to hold the resulting image after filtering.
/// @param buffer buffer image to hold temporal data during function operation.
/// @param maskSize size of mask. Should be 3 (for mask of size 3x3) or 5 (for mask of size 5x5).
/// @see FilterSobelHorizGetBufferSize
void FilterSobelHorizBorder(const QVImage<uChar> &src, QVImage<sShort> &dest, QVImage<uChar> &buffer, uInt maskSize = 3);

/// @brief Applies vertical Sobel filter with border.
///
/// This function is similar to FilterSobelVertMask, but uses negative values for the mask applied,
/// and automatically create a required border and define appropriate pixel values.
///
/// Function FilterSobelNegVertGetBufferSize should be used to initialize sizes for a buffer QVImage<uChar>
/// image, which will hold temporary data for the function operations.
///
/// @param src source image to filter.
/// @param dest destination image to hold the resulting image after filtering.
/// @param buffer buffer image to hold temporal data during function operation.
/// @param maskSize size of mask. Should be 3 (for mask of size 3x3) or 5 (for mask of size 5x5).
/// @see FilterSobelNegVertGetBufferSize
void FilterSobelNegVertBorder(const QVImage<uChar> &src, QVImage<sShort> &dest, QVImage<uChar> &buffer, uInt maskSize = 3);

/// @brief Filters an image using a median filter.
///
/// This function sets each pixel in the output buffer as the median value of all the input pixel
/// values taken in the neighborhood of the processed pixel.
///
/// This filter removes noise and does not cut out signal brightness drops, as an averaging filter does.
///
/// @param src source image to filter.
/// @param dest destination image to hold the resulting image after filtering.
/// @param colMaskSize mask width.
/// @param rowMaskSize mask height.
void FilterMedian(const QVImage<uChar> &src, QVImage<uChar> &dest, uInt colMaskRadius = 3, uInt rowMaskRadius = 3);

/// @brief Filters an image using a median filter.
/// @see FilterMedian(const QVImage<uChar> &, QVImage<uChar> &, uInt, uInt)
void FilterMedian(const QVImage<sShort> &src, QVImage<sShort> &dest, uInt colMaskRadius = 3, uInt rowMaskRadius = 3);

/// @brief Changes an image size.
///
/// This function copies the content of the ROI in the source image, to the ROI of the
/// destination image, adjusting it's original size to make it fit in the destination size
/// It has the effect of resizing the original image, to fit the destination image, if both ROI's
/// hold the whole of their respective images.
///
/// @param src source image to re-size ROI.
/// @param dest destination image to hold the re-sized ROI.
/// @param interpolation interpolation type. Selects the algorithm for interpolate pixel values. Possible values for this parameter are:
///	- IPPI_INTER_NN. Nearest neighbor interpolation
///	- IPPI_INTER_LINEAR. Linear interpolation
///	- IPPI_INTER_CUBIC. Cubic interpolation
///	- IPPI_INTER_SUPER. Supersampling interpolation, cannot be applied for image enlarging
///	- IPPI_INTER_LANCZOS. Interpolation with Lanczos window.
void Resize(const QVImage<uChar> &src, QVImage<uChar> &dest, int interpolation = IPPI_INTER_LINEAR);

/// @brief Changes an image size.
/// @see Resize(const QVImage<uChar> &, QVImage<uChar> &, int)
void Resize(const QVImage<sFloat> &src, QVImage<sFloat> &dest, int interpolation = IPPI_INTER_LINEAR);

/// @brief Allocates size in a buffer image for temporary data for function Canny.
///
/// This function should be applied to any QVImage<uChar> image to be used as a buffer image for function
/// Canny. Initializes dimensions for that buffer image, appropriated for canny algorithm.
///
/// @param src source image to apply later Canny operator.
/// @param buffer buffer image to initialize for further operations.
/// @see Canny
void CannyGetSize(const QVGenericImage &src, QVImage<uChar> &buffer);

/// @brief Allocates size in a buffer image for temporary data for function FilterSobelHoriz.
///
/// This function should be applied to any QVImage<uChar> image to be used as a buffer image for function
/// FilterSobelHoriz. Initializes dimensions for that buffer image, appropriated for that algorithm.
///
/// @param src source image to apply later filter.
/// @param buffer buffer image to initialize for further operations.
/// @param maskSize size of mask. Should be 3 (for mask of size 3x3) or 5 (for mask of size 5x5).
/// @see FilterSobelHoriz
void FilterSobelHorizGetBufferSize(const QVGenericImage &src, QVImage<uChar> &buffer, uInt maskSize = 3);

/// @brief Allocates size in a buffer image for temporary data for function FilterSobelNegVert.
///
/// This function should be applied to any QVImage<uChar> image to be used as a buffer image for function
/// FilterSobelNegVert. Initializes dimensions for that buffer image, appropriated for that algorithm.
///
/// @param src source image to apply later filter.
/// @param buffer buffer image to initialize for further operations.
/// @param maskSize size of mask. Should be 3 (for mask of size 3x3) or 5 (for mask of size 5x5).
/// @see FilterSobelNegVert
void FilterSobelNegVertGetBufferSize(const QVGenericImage &src, QVImage<uChar> &buffer, uInt maskSize = 3);

/// @brief Allocates size in a buffer image for temporary data for function MinEigenValGet.
///
/// This function should be applied to any QVImage<uChar> image to be used as a buffer image for function
/// MinEigenVal. Initializes dimensions for that buffer image, appropriated for that algorithm.
///
/// @param src source image to apply later MinEigenVal function.
/// @param buffer buffer image to initialize for further operations.
/// @param apertureSize Size (in pixels) of the derivative operator used by the function, possible values are 3 or 5.
/// @param avgWindow Size of the blurring window in pixels, possible values are 3 or 5.
/// @see MinEigenVal
void MinEigenValGetBufferSize( const QVGenericImage &src, QVImage<uChar> &buffer, uInt apertureSize = 5, uInt avgWindow = 5);

/// @brief Allocates size in a buffer image for temporary data for function family FloodFill.
///
/// This function should be applied to any QVImage<uChar> image to be used as a buffer image for function
/// family FloodFill. Initializes dimensions for that buffer image, appropriated for that algorithm.
///
/// @param src source image to apply later flood fill function.
/// @param buffer buffer image to initialize for further operations.
/// @see FloodFillRange
void FloodFillGetSize(const QVGenericImage &src, QVImage<uChar> &buffer);

/// @brief Performs Canny algorithm for edge detection.
///
/// This function finds edges in the source image ROI and stores them into the output image
/// dest using the Canny algorithm.
///
/// The function requires a temporary buffer image, that should have been initialized with CannyGetSize function.
///
/// Images dX and dY should be filtered images from the original, from which we desire to obtain the contours, using
/// horizontal and vertical Sobel filters.
///
/// A code example of a full canny extractor, given a source image, is described in the code below:
/// \code
///	[...]
///
///	QVImage<sFloat>	imageFloat(cols, rows), dX(cols, rows), dY(cols, rows);
///	QVImage<uChar>	canny(cols, rows), buffer;
///
///	qvipp::Convert(image, imageFloat);
///	qvipp::FilterSobelHorizMask(imageFloat,dY,3);
///	qvipp::FilterSobelVertMask(imageFloat,dX,3);
///	qvipp::CannyGetSize(canny, buffer);
///	qvipp::Canny(dX, dY, canny, buffer);
///
///	[...]
/// \endcode
///
/// @param dX source image x-derivative.
/// @param dY source image y-derivative.
/// @param dest output image of the detected edges.
/// @param buffer image to hold temporary data during function operations.
/// @param low lower threshold for edges detection.
/// @param high upper threshold for edges detection.
/// @see CannyGetSize
/// @see FilterSobelHoriz
/// @see FilterSobelVert
/// @see FilterSobelHorizBorder
/// @see FilterSobelNegVertBorder
void Canny(QVImage<sShort> &dX, QVImage<sShort> &dY, QVImage<uChar> &dest,
		QVImage<uChar> &buffer, double low = 50.0, double high = 150.0);

/// @brief Performs Canny algorithm for edge detection.
/// @see Canny(QVImage<sShort> &, QVImage<sShort> &, QVImage<uChar> &, QVImage<uChar> &, double, double)
void Canny(QVImage<sFloat> &dX, QVImage<sFloat> &dY, QVImage<uChar> &dest,
		QVImage<uChar> &buffer, double low = 50.0, double high = 150.0);

/// @brief Calculates the minimal eigen value of image blocks for corner detection.
///
/// This function takes a block around the pixel and computes the first derivatives Dx and Dy.
///
/// This operation is performed for every pixel of the image using either Sobel or Scharr kernel
/// in accordance with the kernType parameter.
///
/// The size of the Sobel kernel may be specified the parameter apertureSize.
/// If this parameter is set to 3 - the function used 3x3 kernel, if it set to 5 - the function uses 5x5 kernel.
///
/// Only 3x3 size is available for the Scharr kernel, therefore the parameter apertureSize must be set to
/// 3 if the Scharr kernel is used.
///
/// @param src source image.
/// @param dest destination image to hold the minimal eigen values.
/// @param buffer image to hold temporary data during function operations.
/// @param apertureSize Size (in pixels) of the derivative operator used by the function, possible values are 3 or 5.
/// @param avgWindow Size of the blurring window in pixels, possible values are 3 (3x3 mask size) or 5 (5x5 mask size).
/// @param kernel Specifies the type of kernel used to compute derivatives, possible values are:
///	- ippKernelSobel. Values for aperture size can be 3, or 5.
///	- ippKernelScharr. Value for aperture size can only be 3.
/// @see MinEigenValGetBufferSize
void MinEigenVal(const QVImage<uChar> &src, QVImage<sFloat> &dest, QVImage<uChar> &buffer,
	uInt apertureSize = 3, uInt avgWindow = 5, IppiKernelType kernel = ippKernelSobel);

/// @brief Allocates size in a buffer image for temporary data for function FastMarching.
///
/// This function should be applied to any QVImage<uChar> image to be used as a buffer image for function
/// FastMarching. Initializes dimensions for that buffer image, appropriated for that algorithm.
///
/// @param image source image to apply Fast Marching algorithm.
/// @param buffer buffer image to initialize for further operations.
/// @see FastMarching
void FastMarchingGetBufferSize(const QVGenericImage &image, QVImage<uChar> &buffer);

/// @brief Calculates distance transform to closest zero pixel for all non-zero pixels of source image using fast marching method.
///
/// This function computes the distance from the closest zero pixel to each image pixel according to the Fast Marching
/// Method (FMM). For further info, see:
///
/// A.Telea. An Image Inprinting Technique Based on the Fast Marching Method. Journal of Graphic Tools, vol.9, No.1.
/// ACM Press, 2004.
///
/// @param mask source image to obtain distances to closest zero pixel.
/// @param distances image to hold distances to closest zero pixel.
/// @param radius Radius of the neighborhood of the marked area.
/// @param buffer image to hold temporary data during function operations.
/// @see FastMarchingGetBufferSize
void FastMarching(const QVImage<uChar> &mask, QVImage<sFloat> &distances, int radius, QVImage<uChar> &buffer);

/// @brief Restores unknown image pixels.
///
/// This function reconstructs damaged part of the image, or removes a selected object.
/// The image part to restore is defined by the mask image.
/// Different distant transforms can be used, but the Fast Marching Method (implemented by
/// FastMarching function) provides the best results.
/// The order of pixel restoration is defined by the distance through the initialization the
/// inpainting structure pState by the function ippiInpaintInitAlloc.
/// Pixels are restored in according to the growing of their distance value.
/// When a pixel is inpainted, it is treated as the known one.
/// Two algorithms of direct inpainting are supported
///	- image restoration of the unknown pixel by the weighted sum of approximations by known pixels in the neighborhood.
///	- image restoration based on the Navier-Stokes equations.
///
/// A code example to show the use of this function is below:
/// \code
/// [...]
/// 
/// QVImage<uChar,3> image;	// image to inpaint
/// QVImage<uChar> mask;		// mask, indicating unknown pixels for image (0 known, >0 unknown)
/// 
/// [...]
/// 
/// QVImage<uChar> buffer;		// temporary buffer for FastMarching function
/// QVImage<sFloat> distances(image.getCols(), image.getRows());	// image to hold distances to zero pixels
/// QVImage<uChar,3> inpaint(image.getCols(),image.getRows());	// inpainted image from original
/// 
/// qvipp::FastMarchingGetBufferSize(mask, buffer);		// initiating buffer image for FastMarching function
/// qvipp::FastMarching(mask, distances, radius, buffer);		// calculating distances to zero pixels for image
/// qvipp::Inpaint(image, mask, distances, inpaint, radius);	// inpainting
/// 
/// [...]
/// \endcode
///
/// @param src source image to restore.
/// @param mask mask image. Values different from zero indicate unknown pixels.
/// @param distances image to hold distances to closest zero pixel. Must have been pre-calculated.
/// @param dest destination image to hold restored image.
/// @param radius Radius of the neighborhood of the marked area, used for inpainting.
/// @param flag indicates the algorithm for direct inpainting:
///		- IPP_INPAINT_TELEA. Use weighted sum of approximations by known pixels in the neighborhood.
///		- IPP_INPAINT_NS. Use Navier-Stokes equations.
void Inpaint(const QVImage<uChar> &src, const QVImage<uChar> &mask, const QVImage<sFloat> &distances,
		QVImage<uChar> &dest, int radius = 4,  IppiInpaintFlag flag = IPP_INPAINT_TELEA);

/// @brief Overload for function Inpaint, for different channel number images.
/// @see Inpaint(const QVImage<uChar> &, const QVImage<uChar> &, const QVImage<sFloat> &, QVImage<uChar> &, int, IppiInpaintFlag)
void Inpaint(const QVImage<uChar,3> &src, const QVImage<uChar> &mask, const QVImage<sFloat> &distances,
		QVImage<uChar,3> &dest, int radius = 4,  IppiInpaintFlag flag = IPP_INPAINT_TELEA);

/// @brief Calculates frequency histogram for a given set of values, for a QVImage
///
/// This function computes the intensity histogram of an image in the ranges specified by the QList 'values'.
/// Default value for this QList is empty list, indicating the set of integer which cover the group of valid values for
/// uChar type (from 0 to 255).
///
/// Return value is a QMap variable, that maps pixel values, with their frequency of depicting in the image.
///
/// @param src source image to obtain histogram values.
/// @param values list of QVImage data type values, to obtain frequency in histogram.
QMap<sInt, int> HistogramRange(const QVImage<uChar, 1> &src, QList<uChar> values = QList<uChar>());

/// @brief Performs flood filling of pixels with values in the specified range in the connected area on an image.
///
/// This function performs flood filling of the group of connected pixels in the seed pixel
/// neighborhoods whose pixel values \f$ v_i \f$ satisfies the following condition:
///
/// \f$ v_i - minDelta \leq v_0 \leq v_i + maxDelta \f$
/// 
/// where \f$ v_0 \f$ is the pixel value of the seed point. Values of these pixel is set to the <i>newVal</i> value.
///
/// The function requires a temporary buffer whose size should be computed with the function
/// ippiFloodFillGetSize beforehand.
///
/// @param img image to flood fill.
/// @param buffer temporary buffer image previously initialized with @ref FloodFillGetSize.
/// @param seedX column coordinate for the seed.
/// @param seedY row coordinate for the seed.
/// @param newVal Value to fill with for one-channel data.
/// @param minDelta Minimum difference between neighbor pixels for one-channel data.
/// @param maxDelta Maximum difference between neighbor pixels for one-channel data.
/// @see FloodFillGetSize
void FloodFillRange(QVImage<uChar> &img, QVImage<uChar> &buffer,
	uInt seedX, uInt seedY, uChar newVal, uChar minDelta, uChar maxDelta);

}

#endif
